/********************************************************************************************************
		* PROGRAM      : menus
		* DATE - TIME  : vendredi 01 juin 2007 - 17h27
		* AUTHOR       : gilles bailly ( gilles.bailly enst.fr )
			* FILENAME     : MarkDrawing.cpp
			* LICENSE      : GPL
			* COMMENTARY   :
********************************************************************************************************/
#include  <math.h>
#include <QPainter>
#include <QPainterPath>
#include <QPointF>

#include "MarkDrawing.h"

//=====================
MarkDrawing::MarkDrawing( QObject* parent ){}

//=====================
//MarkDrawing::~MarkDrawing(){}

//======================
int MarkDrawing::orientation( int numItem ){
	int div = numItem /8;
	return numItem - div * 8;
}

//=======================
int MarkDrawing::type( int numItem){
	if(numItem <8){
		return 0; //segment droit
	}else if ( numItem < 24){
		return 1; //courbure
	}else if ( numItem <40){
		return 2; //boucle
	}else if ( numItem < 56 ){
		return 3; //rebroussement
	}else{ return -1; }
}

//======================
int MarkDrawing::sens( int numItem ){
	if(numItem <8){
		return 0;
	}else if (numItem <16 ){
		return 1;
	}else if ( numItem < 24){
		return -1;
	}else if( numItem < 32 ){
		return 1;
	}else if ( numItem <40){
		return -1;
	}else if ( numItem < 48 ){
		return 1;
	}else if ( numItem < 56 ){
		return -1;
	}else{ return -1000; }
}

//======================
int MarkDrawing::getNumItem( int orientation, int type, bool sens){
	if( type == 0 ){
		return orientation ;
	}else{
		return 8 + 16 *( type -1)   + sens * 8 + orientation;
	}
	
	return -1;
}

//======================
void MarkDrawing::drawMark(QPainter& painter, int x, int y, int l, int numItem){
	if(numItem <8){
		drawSimpleMark( painter, x, y, l, numItem);
	}else if ( numItem < 24){
		drawCourbure( painter, x, y, l, numItem);
	}else if ( numItem <40){
		drawInflexion( painter, x, y, l, numItem);
	}else if ( numItem < 56 ){	
		drawPigTail( painter, x, y, l, numItem);
	}
	
	
	
}

//=======================
void MarkDrawing::drawSimpleMark(QPainter& painter, int x, int y, int l, int numItem){
	float r0 = (float)l;
	QPointF center( x, y);
	QPointF vec;
	
	vec.setX( r0 *  ::cos( numItem* 2. * 3.14 / 8.) );
	vec.setY( - r0 *  ::sin( numItem* 2. * 3.14 / 8.) );
	
	painter.drawLine(center, center + vec);
}

//========================
void MarkDrawing::drawCourbure(QPainter& painter, int x, int y, int l, int numItem){
	float r0 = (float) l;
	float r1 = 0.6 * r0 ;
	float r2 = 0.85 * r0 ;
	QPointF center( x, y);
	QPainterPath myPath(center);
	QPointF control1, control2, end0, end1, vec;
	float sens = 1.;
	float diff = 1;
	
	if (numItem >15 && numItem < 24){
		sens = -1.;
		diff = 2.;
	}
	float angle =  (numItem - diff * 8.) * 2. * 3.14 / 8.;
	
	control1.setX(   r1 * ::cos( angle ) );
	control1.setY(  -r1 * ::sin( angle ) );
	control2.setX(   r2*  ::cos( angle ) );
	control2.setY(  -r2 *  ::sin( angle ) );
	
	if( numItem  <24){
		end1.setX(   r0 *  ::cos( (numItem-8)* 2. * 3.14 / 8. +sens *  3.14 / 10.) );
		end1.setY( - r0 *  ::sin( (numItem-8)* 2. * 3.14 / 8. + sens * 3.14 / 10.) );
	}
	
	myPath.cubicTo(center + control1, center + control2, center + end1);
	
	painter.strokePath(myPath, painter.pen());
}




//========================
void MarkDrawing::drawInflexion(QPainter& painter, int x, int y, int l, int numItem){
	float r0 = (float) l;
	float r1 = 0.6 * r0 ;
	//float r2 = 0.85 * r0 ;
	QPointF center( x, y );
	QPainterPath myPath(center);
	QPointF control1, control2, end1, vec;
	float sens = 1.;
	float diff = 3;
	
	if (numItem >31 && numItem < 40){
		sens = -1.;
		diff = 4.;
	}
	float angle =  (numItem - diff * 8.) * 2. * 3.14 / 8.;
	
	control1.setX(   1.9* r0 * ::cos( angle ) );
	control1.setY(  -1.9 * r0 * ::sin( angle ) );
	control2.setX(   r1 *  ::cos( (numItem-8)* 2. * 3.14 / 8. +sens *  3.14 / 8.) );
	control2.setY( - r1 *  ::sin( (numItem-8)* 2. * 3.14 / 8. + sens * 3.14 / 8.) );
	end1.setX(        r1 *  ::cos( (numItem-8)* 2. * 3.14 / 8. +sens *  3.14 / 8.) );
	end1.setY(      - r1 *  ::sin( (numItem-8)* 2. * 3.14 / 8. +sens *  3.14 / 8.) );
	
	myPath.cubicTo(center + control1, center + control2, center + end1);
	
	painter.strokePath(myPath, painter.pen());
}

//=========================
void MarkDrawing::drawPigTail(QPainter& painter, int x , int y , int l , int numItem){
	float r0 = (float) l;
	float r1 = 0.6 * r0 ;
	//float r2 = 0.85 * r0 ;
	QPointF center( x, y );
	QPainterPath myPath(center);
	QPointF control1, control2, end1, vec;
	
	
	float sens = 1.;
	float diff = 5;
	
	if (numItem >=48 && numItem < 56){
		sens = -1.;
		diff = 6.;
	}
	
	float angle =  (numItem - diff * 8.) * 2. * 3.14 / 8.;
	
	control1.setX(   2 * r0 * ::cos( angle ) );
	control1.setY(  -2 * r0 * ::sin( angle ) );
	control2.setX(   r1 *  ::cos( (numItem-8)* 2. * 3.14 / 8. +sens *  3.14 / 4.) );
	control2.setY( - r1 *  ::sin( (numItem-8)* 2. * 3.14 / 8. + sens * 3.14 / 4.) );
	end1.setX(   r1 *  ::cos( (numItem-8)* 2. * 3.14 / 8. ) );
	end1.setY( - r1 *  ::sin( (numItem-8)* 2. * 3.14 / 8. ) );
	
	myPath.cubicTo(center + control1, center + control2, center + end1);
	
	painter.strokePath(myPath, painter.pen());
}

//========================
void MarkDrawing::drawPie( QPainter& painter, int x, int y , int l , int nbSector){
	float r0 = (float) l;
	QPointF center( x, y );
	QPointF vec;
	
	for( int i = 0; i < nbSector; i++){
		vec.setX( r0 *    ::cos( (float) i * 2. * 3.14 / (float)nbSector  + 3.14 / (float)nbSector  ) );
		vec.setY( - r0 *  ::sin( (float) i * 2. * 3.14 / (float) nbSector  + 3.14 /(float) nbSector  )  );
		painter.drawLine(center, center + vec);
	}
	
	painter.drawEllipse( (int)center.x() - l, (int)center.y() - l, 2 *l, 2 * l);
}


//========================
void MarkDrawing::drawCardinal( QPainter& painter, int x, int y , int l , int nbCardinal){
	float r0 = (float) l;
	QPointF center( x, y );
	QPointF vec;
	
	for( int i = 0; i < nbCardinal; i++){
		vec.setX( r0 *    ::cos( (float) i * 2. * 3.14 / (float)nbCardinal   ) );
		vec.setY( - r0 *  ::sin( (float) i * 2. * 3.14 / (float) nbCardinal    )  );
		painter.drawLine(center, center + vec);
	}
	
}








