/********************************************************************************************************
		* PROGRAM      : menus
		* DATE - TIME  : jeudi 26 avril 2007 - 10h17
		* AUTHOR       : gilles bailly ( gilles.bailly@enst.fr )
			* FILENAME     : RadialMenu.h
			* LICENSE      : GPL
			* COMMENTARY   :
********************************************************************************************************/
//RadialMenu.h

#ifndef __RADIAL_MENU__
#define __RADIAL_MENU__

#include<QWidget>
#include <QTimer>


#include <vector>
#include "MenuGestureRecognizer.h"


class QFont;
class StateMachine;
class RadialMenu;
class RadialStateMachine;
class GlassPane;
class Delimiter;


class ShortCutCommand{
public:
	int key;
	int modifier;
	int command;
};

//====================================
class RadialMenuData {
public:
	RadialMenuData();
	float centerDist;
	QFont* font;
	int itemW;
	int itemH;
	
};

//===================================
class RadialMenu:public QWidget{
Q_OBJECT
public:
static QObject* mainWindow;
static RadialMenu* menuForEvent;
static RadialMenuData* defaultRadialMenuData;
static std::string MENU_PATH;

static void setMenuPath( const std::string& _path ){ MENU_PATH = _path; }

friend class RadialStateMachine;
friend class GlassPane;

RadialMenu(QWidget *parent=0);
virtual ~RadialMenu();

//=================
virtual void start(QObject* obj, QEvent* e);
virtual void startShortCut( QKeyEvent* e);
virtual int transformShortCut( QKeyEvent* e );
virtual void stop();
virtual const MenuGestureRecognizer& getRecognizer()const;
virtual void setCenterDist(float);
virtual float getCenterDist();
virtual void openMenu();
virtual void closeMenu();
virtual void initialize(){}
virtual int recognizeGesture();
virtual bool isNoviceMode();
virtual int depth()const;
virtual void showSector(int num, bool novice);
virtual bool contains(int x, int y);
virtual RadialMenu* getMenuEvent(int x, int y);
virtual void setRoot();
virtual void activeFinalItemSelected();
virtual QFont& font();
virtual void setMenuFont(QFont font );
virtual QString getNameOfChild( int id);
virtual void setTraceInNoviceMode( bool b );
virtual void clearTrace();
virtual void abortGesture();
virtual void addShortCutCommand(  const ShortCutCommand& scc );
virtual void menuReset();
virtual void setDelayStart( int _delay );
virtual void setDelayFinish( int _delay );


signals:
void menuPress(int, int);
void menuDrag(int, int);
void menuRelease(int, int);
void menuOpen();
void finalItemSelected(int);
void menuClose();
void menuNoviceMode();


public slots:
virtual void timerCloseOut();
virtual void timerOpenOut();



protected:
virtual void paintEvent(QPaintEvent* e){}
virtual StateMachine* getCurrentState(){ return 0; }
virtual void setDefaultState(){ }
virtual bool eventFilter( QObject *obj, QEvent *event );
//virtual void mouseMoveEvent(QMouseEvent* e);
//virtual void mousePressEvent(QMouseEvent* e);
virtual int getNbSector();
virtual int getNoviceGesture()=0;
virtual int getExpertGesture()=0;

//================
GlassPane* glassPane;
RadialMenuData* rdData;
MenuGestureRecognizer* recognizer;
QTimer timerStart;
int delayStart;
QTimer timerFinish;
int delayFinish;
int curItemSelected;
int lastItemSelected;
bool tracing;
Delimiter* delimiter;
bool traceInNoviceMode ;
std::vector< ShortCutCommand > shortCutCommand;
};



//=================================
class GlassPane:public QWidget{
public:
GlassPane(QWidget* parent = 0);
const RadialMenu& getMenu(){ return *menu; }
void setMenu(RadialMenu* _menu){ menu = _menu; }
void addPoint(QPointF);
void clear();
protected:
virtual void paintEvent(QPaintEvent* e);

private:
RadialMenu* menu;
std::vector<QPointF> trace;

};




#endif


